from distutils.core import Extension
from distutils.command.build_ext import build_ext
from distutils.errors import CompileError
from subprocess import call


ext_modules = [
    Extension(
        "promqlpy._libpromql",
        include_dirs=["go/*"],
        sources=["libpromql.go"],
    ),
]


class BuildFailed(Exception):
    pass


class GoExtBuilder(build_ext):
    def build_extension(self, ext):
        ext_path = self.get_ext_fullpath(ext.name)
        cmd = ["go", "build", "-buildmode=c-shared", "-o", ext_path]
        cmd += ext.sources
        out = call(cmd, cwd="./go")

        if out != 0:
            raise CompileError("Go build failed")


def build(setup_kwargs):
    """
    This function is mandatory in order to build the extensions.
    """
    setup_kwargs.update(
        {"ext_modules": ext_modules, "cmdclass": {"build_ext": GoExtBuilder}}
    )



import cffi
import json
from pathlib import Path
from distutils.sysconfig import get_config_var


here = Path(__file__).absolute().parent
ext_suffix = get_config_var("EXT_SUFFIX")
so_file = str(here / ("_libpromql" + str(ext_suffix)))

ffi = cffi.FFI()

# Load library
lib = ffi.dlopen(so_file)

# Define the function prototypes
ffi.cdef(
    """

struct split_return {
	char* json_result;
	char* err;
};

extern struct split_return split(char* code);
extern void FreeString(char* str);

"""
)


class PromQLException(Exception):
    ...


def split(code: str):
    """
    split PromQL/MetricsQL alert rules into multiple expressions
    """

    result = lib.split(code.encode())
    json_result = ffi.string(result.json_result).decode()
    err = ffi.string(result.err).decode()

    lib.FreeString(result.json_result)
    lib.FreeString(result.err)

    if err:
        raise PromQLException(err)

    return json.loads(json_result)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['promqlpy']

package_data = \
{'': ['*']}

install_requires = \
['cffi>=1.15.1,<2.0.0']

setup_kwargs = {
    'name': 'promqlpy',
    'version': '0.1.0',
    'description': 'Python utils for parsing PromQL/MetricsQL.',
    'long_description': None,
    'author': 'laixintao',
    'author_email': 'laixintaoo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)
